package jalview.datamodel;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertSame;
import jalview.util.MapList;

import org.junit.Test;

public class AlignedCodonFrameTest
{

  /**
   * Test the constructor which copies all except the aligned protein sequences.
   */
  @Test
  public void testConstructor_copyWithSequence()
  {
    AlignedCodonFrame acf = new AlignedCodonFrame(0);
    acf.codons = new int[][]
    { new int[]
    { 1, 3 }, new int[]
    { 4, 6 } };
    MapList map = new MapList(new int[]
    { 1, 3 }, new int[]
    { 1, 1 }, 3, 1);
    SequenceI aaseq = new Sequence("", "FKQ");
    SequenceI dnaseq = new Sequence("", "ATTCGTACGGAC");
    acf.addMap(dnaseq, aaseq, map);
    SequenceI[] newaligned = new SequenceI[1];
    newaligned[0] = new Sequence("", "-F-K-Q");
    newaligned[0].setDatasetSequence(aaseq.getDatasetSequence());
    AlignedCodonFrame copy = new AlignedCodonFrame(acf, newaligned);
    assertSame(copy.codons, acf.codons);
    assertEquals(copy.aaWidth, acf.aaWidth);
    assertSame(copy.getdnaSeqs(), acf.getdnaSeqs());
    assertSame(newaligned[0], copy.getAaForDnaSeq(dnaseq, false));
  }
}
